@@ -12,16 +12,20 @@
 
 #include "settings.h"
 
+#include "action_data/action_data.h"
+#include "action_data/menuentry_shortcut_action_data.h"
+#include "action_data/command_url_shortcut_action_data.h"
+
 #include <kconfig.h>
 #include <kconfiggroup.h>
 #include <kdebug.h>
+#include <khotkeysglobal.h>
 #include <klocale.h>
 #include <kglobal.h>
 #include <kmessagebox.h>
 
-#include "triggers.h"
-#include "conditions.h"
-#include "action_data.h"
+#include "triggers/triggers.h"
+#include "conditions/conditions.h"
 
 namespace KHotKeys
 {
@@ -45,7 +49,7 @@ Settings::~Settings()
     }
 
 
-Action_data_group *Settings::actions()
+ActionDataGroup *Settings::actions()
     {
     return m_actions;
     }
@@ -111,7 +115,7 @@ bool Settings::isDaemonDisabled() const
     }
 
 
-void Settings::setActions( Action_data_group *actions )
+void Settings::setActions( ActionDataGroup *actions )
     {
     delete m_actions;
     m_actions = actions;
@@ -143,9 +147,9 @@ void Settings::setVoiceShortcut( const KShortcut &shortcut )
     }
 
 
-Action_data_group *Settings::takeActions()
+ActionDataGroup *Settings::takeActions()
     {
-    Action_data_group *res = m_actions;
+    ActionDataGroup *res = m_actions;
     m_actions = 0;
     return res;
     }
@@ -159,8 +163,6 @@ KShortcut Settings::voiceShortcut() const
 
 bool Settings::read_settings( bool include_disabled_P )
     {
-    delete m_actions; m_actions = 0;
-
     KConfig cfg( KHOTKEYS_CONFIG_FILE );
     return read_settings( cfg, include_disabled_P, ImportNone );
     }
@@ -170,16 +172,91 @@ bool Settings::import( KConfig& cfg_P, bool ask_P )
     return read_settings( cfg_P, true, ask_P ? ImportAsk : ImportSilent );
     }
 
+
+void Settings::initialize()
+    {
+    // Create the KMenuEdit group
+    get_system_group(ActionDataGroup::SYSTEM_MENUENTRIES);
+    }
+
+
+ActionDataGroup *Settings::get_system_group(ActionDataGroup::system_group_t group_id)
+    {
+    Q_ASSERT(m_actions);
+
+    // Search for the menuentries system group.
+    ActionDataGroup *system_group = NULL;
+
+    Q_FOREACH(KHotKeys::ActionDataBase* element, m_actions->children())
+        {
+        ActionDataGroup *group = dynamic_cast<ActionDataGroup*>(element);
+
+        if (group && (group->system_group() == group_id))
+            {
+            system_group = group;
+            break;
+            }
+        }
+
+    // Check if we found the group
+    if (system_group==NULL)
+        {
+        switch (group_id)
+            {
+            case ActionDataGroup::SYSTEM_MENUENTRIES:
+                system_group = new ActionDataGroup(
+                        m_actions,
+                        "KMenuEdit",
+                        "KMenuEdit Global Shortcuts",
+                        NULL,
+                        ActionDataGroup::SYSTEM_MENUENTRIES,
+                        true);
+                break;
+
+            default:
+                Q_ASSERT(false);
+                return NULL;
+            }
+        }
+
+    Q_ASSERT(system_group);
+    return system_group;
+    }
+
+
 bool Settings::read_settings( KConfig& cfg_P, bool include_disabled_P, ImportType import_P )
     {
-    if( m_actions == NULL )
-        m_actions = new Action_data_group( NULL, "should never see", "should never see",
-            NULL, Action_data_group::SYSTEM_ROOT, true );
-    if( cfg_P.groupList().count() == 0 ) // empty
+    // Reread settings. First delete what we have
+    setActions(NULL);
+
+    // Initialize m_actions
+    m_actions = new ActionDataGroup(
+            NULL,
+            "should never see",
+            "should never see",
+            NULL,
+            ActionDataGroup::SYSTEM_ROOT,
+            true );
+
+    // If the config group we should read from is empty, return.
+    if( cfg_P.groupList().count() == 0 ) {
         return false;
+    }
+
+    // If we read the main settings and there is no main. Initialize the file
+    // and return
     KConfigGroup mainGroup( &cfg_P, "Main" ); // main group
-    if( import_P == ImportNone ) // reading main cfg file
+    if (import_P == ImportNone)
+        {
+        if (!cfg_P.groupList().contains("Main"))
+            {
+            initialize();
+            return false;
+            }
+
+        // List of already imported configuration files
         already_imported = mainGroup.readEntry( "AlreadyImported",QStringList() );
+        }
     else
         {
         QString import_id = mainGroup.readEntry( "ImportId" );
@@ -216,7 +293,7 @@ bool Settings::read_settings( KConfig& cfg_P, bool include_disabled_P, ImportTyp
             read_settings_v2( cfg_P, include_disabled_P );
           break;
         default:
-            kWarning( 1217 ) << "Unknown cfg. file version\n";
+            kWarning() << "Unknown cfg. file version\n";
           return false;
         case -1234576: // no config file
             if( import_P ) // if importing, this is an error
@@ -236,29 +313,27 @@ bool Settings::read_settings( KConfig& cfg_P, bool include_disabled_P, ImportTyp
     gestures_exclude = new Windowdef_list( gesturesExcludeConfig );
     KConfigGroup voiceConfig( &cfg_P, "Voice" );
     voice_shortcut=KShortcut( voiceConfig.readEntry("Shortcut" , "")  );
+
+    // Ensure the system groups exist
+    initialize();
     return true;
     }
 
-void Settings::write_settings( Action_data_group *action_list )
+void Settings::write_settings()
     {
     KConfig cfg( KHOTKEYS_CONFIG_FILE );
 
-    if (action_list==0)
-        {
-        action_list = m_actions;
-        }
-
 // CHECKME    smazat stare sekce ?
     QStringList groups = cfg.groupList();
-    for( QStringList::ConstIterator it = groups.begin();
-         it != groups.end();
+    for( QStringList::ConstIterator it = groups.constBegin();
+         it != groups.constEnd();
          ++it )
         cfg.deleteGroup( *it );
     KConfigGroup mainGroup( &cfg, "Main" ); // main group
     mainGroup.writeEntry( "Version", 2 ); // now it's version 2 cfg. file
     mainGroup.writeEntry( "AlreadyImported", already_imported );
     KConfigGroup dataGroup( &cfg,  "Data" );
-    int cnt = write_actions_recursively_v2( dataGroup, action_list, true );
+    int cnt = write_actions_recursively_v2( dataGroup, m_actions, true );
     mainGroup.writeEntry( "Autostart", cnt != 0 && !daemon_disabled );
     mainGroup.writeEntry( "Disabled", daemon_disabled );
     KConfigGroup gesturesConfig( &cfg, "Gestures" );
@@ -280,25 +355,23 @@ void Settings::write_settings( Action_data_group *action_list )
 
 // return value means the number of enabled actions written in the cfg file
 // i.e. 'Autostart' for value > 0 should be on
-int Settings::write_actions_recursively_v2( KConfigGroup& cfg_P, Action_data_group* parent_P, bool enabled_P )
+int Settings::write_actions_recursively_v2( KConfigGroup& cfg_P, ActionDataGroup* parent_P, bool enabled_P )
     {
     int enabled_cnt = 0;
     QString save_cfg_group = cfg_P.name();
     int cnt = 0;
     if( parent_P )
         {
-        for( Action_data_group::ConstIterator it = parent_P->first_child();
-            it != parent_P->after_last_child();
-             ++it )
+        Q_FOREACH(ActionDataBase *child,parent_P->children())
             {
             ++cnt;
-            if( enabled_P && (*it)->enabled( true ))
+            if( enabled_P && child->enabled( true ))
                 ++enabled_cnt;
             KConfigGroup itConfig( cfg_P.config(), save_cfg_group + '_' + QString::number( cnt ));
-            ( *it )->cfg_write( itConfig );
-            Action_data_group* grp = dynamic_cast< Action_data_group* >( *it );
+            child->cfg_write( itConfig );
+            ActionDataGroup* grp = dynamic_cast< ActionDataGroup* >(child);
             if( grp != NULL )
-                enabled_cnt += write_actions_recursively_v2( itConfig, grp, enabled_P && (*it)->enabled( true ));
+                enabled_cnt += write_actions_recursively_v2( itConfig, grp, enabled_P && child->enabled( true ));
             }
         }
     cfg_P.writeEntry( "DataCount", cnt );
@@ -311,7 +384,7 @@ void Settings::read_settings_v2( KConfig& cfg_P, bool include_disabled_P  )
     read_actions_recursively_v2( dataGroup, m_actions, include_disabled_P );
     }
 
-void Settings::read_actions_recursively_v2( KConfigGroup& cfg_P, Action_data_group* parent_P,
+void Settings::read_actions_recursively_v2( KConfigGroup& cfg_P, ActionDataGroup* parent_P,
     bool include_disabled_P )
     {
     QString save_cfg_group = cfg_P.name();
@@ -321,10 +394,10 @@ void Settings::read_actions_recursively_v2( KConfigGroup& cfg_P, Action_data_gro
          ++i )
         {
         KConfigGroup itConfig( cfg_P.config(), save_cfg_group + '_' + QString::number( i ));
-        if( include_disabled_P || Action_data_base::cfg_is_enabled( itConfig ))
+        if( include_disabled_P || ActionDataBase::cfg_is_enabled( itConfig ))
             {
-            Action_data_base* new_action = Action_data_base::create_cfg_read( itConfig, parent_P );
-            Action_data_group* grp = dynamic_cast< Action_data_group* >( new_action );
+            ActionDataBase* new_action = ActionDataBase::create_cfg_read( itConfig, parent_P );
+            ActionDataGroup* grp = dynamic_cast< ActionDataGroup* >( new_action );
             if( grp != NULL )
                 read_actions_recursively_v2( itConfig, grp, include_disabled_P );
             }
@@ -336,15 +409,13 @@ void Settings::read_settings_v1( KConfig& cfg_P )
     {
     KConfigGroup mainGroup( &cfg_P, "Main" );
     int sections = mainGroup.readEntry( "Num_Sections", 0 );
-    Action_data_group* menuentries = NULL;
-    for( Action_data_group::ConstIterator it = m_actions->first_child();
-         it != m_actions->after_last_child();
-         ++it )
+    ActionDataGroup* menuentries = NULL;
+    Q_FOREACH(ActionDataBase *child, m_actions->children())
         {
-        Action_data_group* tmp = dynamic_cast< Action_data_group* >( *it );
+        ActionDataGroup* tmp = dynamic_cast< ActionDataGroup* >(child);
         if( tmp == NULL )
             continue;
-        if( tmp->system_group() == Action_data_group::SYSTEM_MENUENTRIES )
+        if( tmp->system_group() == ActionDataGroup::SYSTEM_MENUENTRIES )
             {
             menuentries = tmp;
             break;
@@ -373,18 +444,18 @@ void Settings::read_settings_v1( KConfig& cfg_P )
             {
             if( menuentries == NULL )
                 {
-                menuentries = new Action_data_group( m_actions,
+                menuentries = new ActionDataGroup( m_actions,
                     i18n( MENU_EDITOR_ENTRIES_GROUP_NAME ),
                     i18n( "These entries were created using Menu Editor." ), NULL,
-                    Action_data_group::SYSTEM_MENUENTRIES, true );
+                    ActionDataGroup::SYSTEM_MENUENTRIES, true );
                 menuentries->set_conditions( new Condition_list( "", menuentries ));
                 }
-            ( void ) new Menuentry_shortcut_action_data( menuentries, name, "",
+            ( void ) new MenuEntryShortcutActionData( menuentries, name, "",
                 KShortcut( shortcut ), run );
             }
         else
             {
-            ( void ) new Command_url_shortcut_action_data( m_actions, name, "",
+            ( void ) new CommandUrlShortcutActionData( m_actions, name, "",
                 KShortcut( shortcut ), run );
             }
         }
