@@ -41,14 +41,15 @@
 
 #include "resultitem.h"
 
-ResultScene::ResultScene(Plasma::RunnerManager *manager, QObject *parent)
+ResultScene::ResultScene(Plasma::RunnerManager *manager, QWidget * focusBase, QObject *parent)
     : QGraphicsScene(parent),
       m_runnerManager(manager),
       m_cIndex(0),
       m_rowStride(0),
       m_pageStride(0),
       m_pageCount(0),
-      m_currentPage(0)
+      m_currentPage(0),
+      m_focusBase(focusBase)
 {
     setItemIndexMethod(NoIndex);
 
@@ -122,6 +123,15 @@ void ResultScene::clearMatches()
     emit matchCountChanged(0);
 }
 
+bool ResultScene::canMoveItemFocus() const
+{
+    // We prevent a late query result from stealing the item focus from the user
+    // The item focus can be moved only if the focusBase widget (the khistorycombobox)
+    // has focus (i.e. the user is still typing or waiting) or if there is no item currently focused
+
+    return m_focusBase->hasFocus() || (!focusItem());
+}
+
 void ResultScene::setQueryMatches(const QList<Plasma::QueryMatch> &m)
 {
     // kDebug() << "============================" << endl << "matches retrieved: " << m.count();
@@ -184,15 +194,15 @@ void ResultScene::setQueryMatches(const QList<Plasma::QueryMatch> &m)
         item->setIndex(i);
 
         // it is vital that focus is set *after* the index
-        if (i == 0) {
+        if (i == 0 && canMoveItemFocus()) {
             setFocusItem(item);
+            emit itemHoverEnter(item);
         }
 
         ++i;
         tab = item;
     }
 
-    emit itemHoverEnter(m_items.at(0));
 }
 
 ResultItem* ResultScene::addQueryMatch(const Plasma::QueryMatch &match, bool useAnyId)
@@ -323,14 +333,14 @@ void ResultScene::slotArrowResultItemReleased()
 
 bool ResultScene::launchQuery(const QString &term)
 {
-    bool temp = !(term.isEmpty() || m_runnerManager->query() == term);
+    bool temp = !(term.trimmed().isEmpty() || m_runnerManager->query() == term.trimmed());
     m_runnerManager->launchQuery(term);
     return temp;
 }
 
 bool ResultScene::launchQuery(const QString &term, const QString &runner)
 {
-    bool temp = !(term.isEmpty() || m_runnerManager->query() == term);
+    bool temp = !(term.trimmed().isEmpty() || m_runnerManager->query() == term.trimmed());
     m_runnerManager->launchQuery(term, runner);
     return temp;
 }
