@@ -59,7 +59,7 @@ KDecorationOptions* KDecoration::options_;
 
 KDecoration::KDecoration( KDecorationBridge* bridge, KDecorationFactory* factory )
     :   bridge_( bridge ),
-	w_( NULL ),
+        w_( NULL ),
         factory_( factory )
     {
     factory->addDecoration( this );
@@ -382,6 +382,65 @@ KDecoration::Position KDecoration::mousePosition( const QPoint& p ) const
     return m;
     }
 
+
+KDecorationUnstable::KDecorationUnstable( KDecorationBridge* bridge, KDecorationFactory* factory )
+    : KDecoration( bridge, factory )
+    {
+    Q_ASSERT( dynamic_cast< KDecorationBridgeUnstable* >( bridge ));
+    }
+
+KDecorationUnstable::~KDecorationUnstable()
+    {
+    }
+
+QList<QRect> KDecorationUnstable::shadowQuads( ShadowType type ) const
+    {
+    Q_UNUSED( type );
+    return QList<QRect>();
+    }
+
+double KDecorationUnstable::shadowOpacity( ShadowType type ) const
+    {
+    if( isActive() && type == ShadowBorderedActive )
+        return 1.0;
+    else if( !isActive() && type == ShadowBorderedInactive )
+        return 1.0;
+    return 0.0;
+    }
+
+double KDecorationUnstable::shadowBrightness( ShadowType type ) const
+    {
+    Q_UNUSED( type );
+    return 1.0;
+    }
+
+double KDecorationUnstable::shadowSaturation( ShadowType type ) const
+    {
+    Q_UNUSED( type );
+    return 1.0;
+    }
+
+void KDecorationUnstable::repaintShadow()
+    {
+    static_cast< KDecorationBridgeUnstable* >( bridge_ )->repaintShadow();
+    }
+
+bool KDecorationUnstable::compositingActive() const
+    {
+    return static_cast< KDecorationBridgeUnstable* >( bridge_ )->compositingActive();
+    }
+
+bool KDecorationUnstable::shadowsActive() const
+    {
+    return static_cast< KDecorationBridgeUnstable* >( bridge_ )->shadowsActive();
+    }
+
+double KDecorationUnstable::opacity() const
+    {
+    return static_cast< KDecorationBridgeUnstable* >( bridge_ )->opacity();
+    }
+
+
 KDecorationOptions::KDecorationOptions()
     : d( new KDecorationOptionsPrivate )
     {
@@ -417,7 +476,7 @@ QPalette KDecorationOptions::palette(ColorType type, bool active) const
 #ifdef __GNUC__
 #warning KDE4 : why construct the palette this way?
 #endif    
-		// TODO: Is this worth 'porting' to Qt4?
+                // TODO: Is this worth 'porting' to Qt4?
 //     d->pal[idx] = new QPalette(Qt::black, d->colors[idx], d->colors[idx].light(150),
 //                               d->colors[idx].dark(), d->colors[idx].dark(120),
 //                               Qt::black, QApplication::palette().active().
