@@ -26,13 +26,18 @@
 
 // Qt
 #include <QTimer>
+#include <QSize>
 
 // KDE
 #include <taskmanager/taskmanager.h>
+#include <taskmanager/abstractgroupableitem.h>
+#include <taskmanager/groupmanager.h>
+#include <taskmanager/taskitem.h>
+#include <taskmanager/startup.h>
 #include <KDialog>
 
 // Plasma
-#include <plasma/applet.h>
+#include <Plasma/Applet>
 
 class QGraphicsLinearLayout;
 
@@ -41,13 +46,22 @@ class KColorScheme;
 namespace Plasma
 {
     class LayoutAnimator;
-    class PanelSvg;
+    class FrameSvg;
 } // namespace Plasma
 
 class WindowTaskItem;
+class TaskGroupItem;
+class AbstractTaskItem;
 
 using TaskManager::StartupPtr;
 using TaskManager::TaskPtr;
+using TaskManager::StartupPtr;
+using TaskManager::GroupPtr;
+using TaskManager::AbstractItemPtr;
+using TaskManager::AbstractGroupableItem;
+using TaskManager::GroupManager;
+using TaskManager::TaskItem;
+using TaskManager::TaskGroup;
 
 /**
  * An applet which provides a visual representation of running
@@ -71,8 +85,7 @@ public:
 
         void constraintsEvent(Plasma::Constraints constraints);
 
-        Plasma::PanelSvg *itemBackground();
-        QPixmap *taskAlphaPixmap(const QSize &size);
+        Plasma::FrameSvg *itemBackground();
         KColorScheme *colorScheme();
 
         qreal itemLeftMargin() { return m_leftMargin; }
@@ -81,62 +94,65 @@ public:
         qreal itemBottomMargin() { return m_bottomMargin; }
         void resizeItemBackground(const QSizeF &newSize);
 
+        TaskGroupItem*  rootGroupItem();
+
+        TaskManager::GroupManager &groupManager() const;
+
+        Qt::KeyboardModifiers groupModifierKey() const;
+
+        bool showTooltip() const;
+
+
+signals:
+    /**
+    * emitted whenever we receive a constraintsEvent
+    */
+    void constraintsChanged(Plasma::Constraints);
+
+    void desktopChanged(int,int);
+    void settingsChanged();
+
 protected slots:
         void configAccepted();
-        virtual void wheelEvent(QGraphicsSceneWheelEvent *);
         void themeRefresh();
-        void updateActive(WindowTaskItem *task);
 
 protected:
         void createConfigurationInterface(KConfigDialog *parent);
+        QSizeF sizeHint(Qt::SizeHint which, const QSizeF & constraint = QSizeF()) const;
+        void adjustGroupingStrategy();
 
 private slots:
-        void addWindowTask(TaskPtr);
-        void removeWindowTask(TaskPtr);
-
-        void addStartingTask(StartupPtr);
-        void removeStartingTask(StartupPtr);
-
-        void currentDesktopChanged(int);
-        void taskMovedDesktop(TaskPtr task);
-        void windowChangedGeometry(TaskPtr task);
-        void checkScreenChange();
+        /**
+        * Somthing has changed in the tree of the GroupingStrategy
+        */
+        void reload();
+        //void groupRemoved(TaskGroup *item);
+        void changeSizeHint(Qt::SizeHint which);
+        void dialogGroupingChanged(int index);
 
 private:
-        // creates task representations for existing windows
-        // and sets up connections to listen for addition or removal
-        // of windows
-        void registerWindowTasks();
-
-        // remove all tasks from the taskbar
-        void removeAllWindowTasks();
-
-        void insertItemBeforeSpacer(QGraphicsWidget * item);
-        void adjustStretch();
-
-        bool isOnMyScreen(TaskPtr task);
-        void reconnect();
-
-        QHash<TaskPtr,WindowTaskItem*> m_windowTaskItems;
-        QHash<StartupPtr,WindowTaskItem*> m_startupTaskItems;
-        QHash<TaskPtr,WindowTaskItem*>::iterator m_activeTask;
-
         bool m_showTooltip;
-        bool m_showOnlyCurrentDesktop;
-        bool m_showOnlyCurrentScreen;
-        QGraphicsLinearLayout *m_layout;
-        Ui::tasksConfig m_ui;
-        QList<TaskPtr> m_geometryTasks;
-        QTimer m_screenTimer;
-        QGraphicsWidget *m_spacer;
+        Plasma::LayoutAnimator *m_animator;
+        QGraphicsLinearLayout *layout;
 
-        Plasma::PanelSvg *m_taskItemBackground;
-        QPixmap *m_taskAlphaPixmap;
+        Ui::tasksConfig m_ui;
+        QTimer m_screenTimer;
+
+        Plasma::FrameSvg *m_taskItemBackground;
         KColorScheme *m_colorScheme;
         qreal m_leftMargin;
         qreal m_topMargin;
         qreal m_rightMargin;
         qreal m_bottomMargin;
+
+        TaskGroupItem *m_rootGroupItem;
+        GroupManager *m_groupManager;
+        TaskManager::GroupManager::TaskGroupingStrategy m_groupingStrategy;
+        bool m_groupWhenFull;
+        Qt::KeyboardModifier m_groupModifierKey;
+
+        int m_currentDesktop;
+
 };
 
 #endif
