@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2007 by Shawn Starr <shawn.starr@rogers.com>            *
+ *   Copyright (C) 2007-2008 by Shawn Starr <shawn.starr@rogers.com>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -24,8 +24,12 @@
 class NOAAIon::Private : public QObject
 {
 public:
-    Private() { m_url=0; }
-    ~Private() { delete m_url; }
+    Private() {
+        m_url = 0;
+    }
+    ~Private() {
+        delete m_url;
+    }
 
 private:
     struct XMLMapInfo {
@@ -53,13 +57,42 @@ public:
     KUrl *m_url;
     KIO::TransferJob *m_job;
 
-    int m_timezoneType;  // Ion option: Timezone may be local time or UTC time
-    int m_measureType; // Ion option: Units may be Metric or Imperial
-    bool m_windInMeters; // Ion option: Display wind format in meters per second only
-    bool m_windInKnots; // Ion option: Display wind format in knots
-    bool m_windInBft; // ion option: Display wind by the beaufort scale model
+    QDateTime m_dateFormat;
 };
 
+QMap<QString, IonInterface::WindDirections> NOAAIon::setupWindIconMappings(void)
+{
+    QMap<QString, WindDirections> windDir;
+    windDir["north"] = N;
+    windDir["northeast"] = NE;
+    windDir["south"] = S;
+    windDir["southwest"] = SW;
+    windDir["east"] = E;
+    windDir["southeast"] = SE;
+    windDir["west"] = W;
+    windDir["northwest"] = NW;
+    windDir["calm"] = VR;
+    return windDir;
+}
+
+QMap<QString, IonInterface::ConditionIcons> NOAAIon::setupConditionIconMappings(void)
+{
+
+    QMap<QString, ConditionIcons> conditionList;
+    return conditionList;
+}
+
+QMap<QString, IonInterface::ConditionIcons> const& NOAAIon::conditionIcons(void)
+{
+    static QMap<QString, ConditionIcons> const condval = setupConditionIconMappings();
+    return condval;
+}
+
+QMap<QString, IonInterface::WindDirections> const& NOAAIon::windIcons(void)
+{
+    static QMap<QString, WindDirections> const wval = setupWindIconMappings();
+    return wval;
+}
 
 // ctor, dtor
 NOAAIon::NOAAIon(QObject *parent, const QVariantList &args)
@@ -87,7 +120,7 @@ QStringList NOAAIon::validate(const QString& source) const
     QHash<QString, QString>::const_iterator it = d->m_locations.constBegin();
     while (it != d->m_locations.constEnd()) {
         if (it.value().toLower().contains(source.toLower())) {
-            placeList.append(QString("place|%1").arg(it.value().split("|")[1]));
+            placeList.append(QString("place|%1").arg(it.value().split('|')[1]));
         }
         ++it;
     }
@@ -107,8 +140,14 @@ bool NOAAIon::updateIonSource(const QString& source)
     // ionname:validate:place_name - Triggers validation of place
     // ionname:weather:place_name - Triggers receiving weather of place
 
-    kDebug() << "updateIonSource() SOURCE: " << source;
     QStringList sourceAction = source.split('|');
+
+    // Guard: if the size of array is not 2 then we have bad data, return an error
+    if (sourceAction.size() < 2) {
+        setData(source, "validate", QString("noaa|timeout"));
+        return true;
+    }
+    
     if (sourceAction[1] == QString("validate")) {
         kDebug() << "Initiate Validating of place: " << sourceAction[2];
         QStringList result = validate(QString("%1|%2").arg(sourceAction[0]).arg(sourceAction[2]));
@@ -151,11 +190,18 @@ void NOAAIon::getXMLData(const QString& source)
     KUrl url;
 
     QString dataKey = source;
-    dataKey.replace("|weather", "");
+    dataKey.remove("|weather");
     url = d->m_place[dataKey].XMLurl;
 
     kDebug() << "URL Location: " << url.url();
 
+    // If this is empty we have no valid data, send out an error and abort.
+    //
+    if (url.url().isEmpty()) { 
+        setData(source, "validate", QString("noaa|timeout"));
+        return;
+    }
+
     d->m_job = KIO::get(url.url(), KIO::Reload, KIO::HideProgressInfo);
     d->m_jobXml.insert(d->m_job, new QXmlStreamReader);
     d->m_jobList.insert(d->m_job, source);
@@ -181,7 +227,6 @@ void NOAAIon::setup_slotDataArrived(KIO::Job *job, const QByteArray &data)
 
 void NOAAIon::slotDataArrived(KIO::Job *job, const QByteArray &data)
 {
-
     if (data.isEmpty() || !d->m_jobXml.contains(job)) {
         return;
     }
@@ -300,8 +345,15 @@ WeatherData NOAAIon::parseWeatherSite(WeatherData& data, QXmlStreamReader& xml)
                 data.stationID = xml.readElementText();
             } else if (xml.name() == "observation_time") {
                 data.observationTime = xml.readElementText();
+                QStringList tmpDateStr = data.observationTime.split(' ');
+                data.observationTime = QString("%1 %2").arg(tmpDateStr[5]).arg(tmpDateStr[6]);
+                d->m_dateFormat = QDateTime::fromString(data.observationTime, "h:mm ap");
+                data.iconPeriodHour = d->m_dateFormat.toString("HH");
+                data.iconPeriodAP = d->m_dateFormat.toString("ap");
+
             } else if (xml.name() == "weather") {
                 data.weather = xml.readElementText();
+                // Pick which icon set depending on period of day
             } else if (xml.name() == "temp_f") {
                 data.temperature_F = xml.readElementText();
             } else if (xml.name() == "temp_c") {
@@ -381,38 +433,6 @@ void NOAAIon::parseUnknownElement(QXmlStreamReader& xml)
     }
 }
 
-void NOAAIon::setMeasureUnit(const QString& unitType)
-{
-    d->m_measureType = unitType.toInt();
-}
-
-// Not used in this ion yet.
-void NOAAIon::setTimezoneFormat(const QString& tz)
-{
-    d->m_timezoneType = tz.toInt(); // Boolean
-}
-
-bool NOAAIon::metricUnit()
-{
-    if (d->m_measureType == KLocale::Metric) {
-        return true;
-    }
-
-    // Imperial units
-    return false;
-}
-
-// Not used in this ion yet.
-bool NOAAIon::timezone()
-{
-    if (d->m_timezoneType) {
-        return true;
-    }
-
-    // Not UTC, local time
-    return false;
-}
-
 void NOAAIon::updateWeather(const QString& source)
 {
     QMap<QString, QString> dataFields;
@@ -425,6 +445,22 @@ void NOAAIon::updateWeather(const QString& source)
     // Real weather - Current conditions
     setData(source, "Observation Period", observationTime(source));
     setData(source, "Current Conditions", condition(source));
+
+// FIXME: We'll need major fuzzy logic, this isn't pretty: http://www.weather.gov/xml/current_obs/weather.php
+    //QMap<QString, ConditionIcons> conditionList;
+    //conditionList = conditionIcons();
+
+/*
+    if ((periodHour(source) >= 0 && periodHour(source) < 6) || (periodHour(source) >= 18)) {
+        // Night
+        // - Fill in condition fuzzy logic 
+    } else {
+        // Day
+        // - Fill in condition fuzzy logic
+    }
+*/
+    setData(source, "Condition Icon", "weather-none-available");
+
     dataFields = temperature(source);
     setData(source, "Temperature", dataFields["temperature"]);
 
@@ -435,11 +471,11 @@ void NOAAIon::updateWeather(const QString& source)
     // Do we have a comfort temperature? if so display it
     if (dataFields["comfortTemperature"] != "N/A") {
         if (d->m_weatherData[source].windchill_F != "NA") {
-            setData(source, "Windchill", QString("%1%2").arg(dataFields["comfortTemperature"]).arg(QChar(176)));
+            setData(source, "Windchill", QString("%1").arg(dataFields["comfortTemperature"]));
             setData(source, "Humidex", "N/A");
         }
         if (d->m_weatherData[source].heatindex_F != "NA" && d->m_weatherData[source].temperature_F.toInt() != d->m_weatherData[source].heatindex_F.toInt()) {
-            setData(source, "Humidex", QString("%1%2").arg(dataFields["comfortTemperature"]).arg(QChar(176)));
+            setData(source, "Humidex", QString("%1").arg(dataFields["comfortTemperature"]));
             setData(source, "Windchill", "N/A");
         }
     } else {
@@ -468,6 +504,9 @@ void NOAAIon::updateWeather(const QString& source)
 
     setData(source, "Humidity", humidity(source));
 
+    // Set number of forecasts per day/night supported, none for this ion right now
+    setData(source, QString("Total Weather Days"), 0);
+
     dataFields = wind(source);
     setData(source, "Wind Speed", dataFields["windSpeed"]);
 
@@ -477,7 +516,7 @@ void NOAAIon::updateWeather(const QString& source)
 
     setData(source, "Wind Gust", dataFields["windGust"]);
     setData(source, "Wind Gust Unit", dataFields["windGustUnit"]);
-    setData(source, "Wind Direction", dataFields["windDirection"]);
+    setData(source, "Wind Direction", getWindDirectionIcon(windIcons(), dataFields["windDirection"].toLower()));
     setData(source, "Credit", "Data provided by NOAA National Weather Service");
 }
 
@@ -499,6 +538,22 @@ QString NOAAIon::observationTime(const QString& source)
 {
     return d->m_weatherData[source].observationTime;
 }
+
+/*
+bool NOAAIon::night(const QString& source)
+{
+    if (d->m_weatherData[source].iconPeriodAP == "pm") {
+        return true;
+    }
+    return false;
+}
+*/
+
+int NOAAIon::periodHour(const QString& source)
+{
+    return d->m_weatherData[source].iconPeriodHour.toInt();
+}
+
 QString NOAAIon::condition(const QString& source)
 {
     if (d->m_weatherData[source].weather.isEmpty() || d->m_weatherData[source].weather == "NA") {
@@ -509,9 +564,6 @@ QString NOAAIon::condition(const QString& source)
 
 QString NOAAIon::dewpoint(const QString& source)
 {
-    if (metricUnit()) {
-        return d->m_weatherData[source].dewpoint_C;
-    }
     return d->m_weatherData[source].dewpoint_F;
 }
 
@@ -531,41 +583,24 @@ QMap<QString, QString> NOAAIon::visibility(const QString& source)
         visibilityInfo.insert("visibility", QString("N/A"));
         return visibilityInfo;
     }
-    if (metricUnit()) {
-        visibilityInfo.insert("visibility", QString::number(WeatherFormula::milesToKM(d->m_weatherData[source].visibility.toFloat()), 'f', 1));
-        visibilityInfo.insert("visibilityUnit", "km");
-        return visibilityInfo;
-    }
     visibilityInfo.insert("visibility", d->m_weatherData[source].visibility);
-    visibilityInfo.insert("visibilityUnit", "mi");
+    visibilityInfo.insert("visibilityUnit", QString::number(WeatherUtils::Miles));
     return visibilityInfo;
 }
 
 QMap<QString, QString> NOAAIon::temperature(const QString& source)
 {
     QMap<QString, QString> temperatureInfo;
-    if (metricUnit()) {
-        temperatureInfo.insert("temperature", d->m_weatherData[source].temperature_C);
-        temperatureInfo.insert("temperatureUnit", QString("%1C").arg(QChar(176)));
-    } else {
-        temperatureInfo.insert("temperature", d->m_weatherData[source].temperature_F);
-        temperatureInfo.insert("temperatureUnit", QString("%1F").arg(QChar(176)));
-    }
+    temperatureInfo.insert("temperature", d->m_weatherData[source].temperature_F);
+    temperatureInfo.insert("temperatureUnit", QString::number(WeatherUtils::Fahrenheit));
     temperatureInfo.insert("comfortTemperature", "N/A");
 
     if (d->m_weatherData[source].heatindex_F != "NA" && d->m_weatherData[source].windchill_F == "NA") {
-        if (metricUnit()) {
-            temperatureInfo.insert("comfortTemperature", d->m_weatherData[source].heatindex_C);
-        } else {
-            temperatureInfo.insert("comfortTemperature", d->m_weatherData[source].heatindex_F);
-        }
+        temperatureInfo.insert("comfortTemperature", d->m_weatherData[source].heatindex_F);
     }
+
     if (d->m_weatherData[source].windchill_F != "NA" && d->m_weatherData[source].heatindex_F == "NA") {
-        if (metricUnit()) {
-            temperatureInfo.insert("comfortTemperature", d->m_weatherData[source].windchill_C);
-        } else {
-            temperatureInfo.insert("comfortTemperature", d->m_weatherData[source].windchill_F);
-        }
+        temperatureInfo.insert("comfortTemperature", d->m_weatherData[source].windchill_F);
     }
 
     return temperatureInfo;
@@ -578,13 +613,9 @@ QMap<QString, QString> NOAAIon::pressure(const QString& source)
         pressureInfo.insert("pressure", "N/A");
         return pressureInfo;
     }
-    if (metricUnit()) {
-        pressureInfo.insert("pressure", QString::number(WeatherFormula::inchesToKilopascals(d->m_weatherData[source].pressure.toFloat()), 'f', 1));
-        pressureInfo.insert("pressureUnit", "kPa");
-    } else {
-        pressureInfo.insert("pressure", d->m_weatherData[source].pressure);
-        pressureInfo.insert("pressureUnit", "in");
-    }
+
+    pressureInfo.insert("pressure", d->m_weatherData[source].pressure);
+    pressureInfo.insert("pressureUnit", QString::number(WeatherUtils::InchesHG));
     return pressureInfo;
 }
 
@@ -595,29 +626,19 @@ QMap<QString, QString> NOAAIon::wind(const QString& source)
     // May not have any winds
     if (d->m_weatherData[source].windSpeed == "NA") {
         windInfo.insert("windSpeed", "Calm");
-        windInfo.insert("windUnit", "N/A");
+        windInfo.insert("windUnit", QString::number(WeatherUtils::NoUnit));
     } else {
-        if (metricUnit()) {
-                windInfo.insert("windSpeed", QString::number(WeatherFormula::milesToKM(d->m_weatherData[source].windSpeed.toFloat()), 'f', 1));
-                windInfo.insert("windUnit", "km/h");
-        } else {
-                windInfo.insert("windSpeed", QString::number(d->m_weatherData[source].windSpeed.toFloat(), 'f', 1));
-                windInfo.insert("windUnit", "mph");
-        }
+        windInfo.insert("windSpeed", QString::number(d->m_weatherData[source].windSpeed.toFloat(), 'f', 1));
+        windInfo.insert("windUnit", QString::number(WeatherUtils::MilesAnHour));
     }
 
     // May not always have gusty winds
     if (d->m_weatherData[source].windGust == "NA") {
         windInfo.insert("windGust", "N/A");
-        windInfo.insert("windGustUnit", "N/A");
+        windInfo.insert("windGustUnit", QString::number(WeatherUtils::NoUnit));
     } else {
-        if (metricUnit()) {
-                windInfo.insert("windGust", QString::number(WeatherFormula::milesToKM(d->m_weatherData[source].windGust.toFloat()), 'f', 1));
-                windInfo.insert("windGustUnit", "km/h");
-        } else {
-                windInfo.insert("windGust", QString::number(d->m_weatherData[source].windGust.toFloat(), 'f', 1));
-                windInfo.insert("windGustUnit", "mph");
-          }
+        windInfo.insert("windGust", QString::number(d->m_weatherData[source].windGust.toFloat(), 'f', 1));
+        windInfo.insert("windGustUnit", QString::number(WeatherUtils::MilesAnHour));
     }
 
     if (d->m_weatherData[source].windDirection.isEmpty()) {
