@@ -498,6 +498,10 @@ module Plasma
       end
     end
 
+    def set_content(widget, data)
+      setContent(widget, data)
+    end
+
     def clearContent(widget)
       if widget.kind_of?(PlasmaScripting::Applet)
         super(widget.applet_script.applet)
@@ -506,6 +510,10 @@ module Plasma
       end
     end
 
+    def clear_content(widget)
+      clearContent(widget)
+    end
+
     def show(widget)
       if widget.kind_of?(PlasmaScripting::Applet)
         super(widget.applet_script.applet)
@@ -534,6 +542,10 @@ module Plasma
       end
     end
 
+    def register_widget(widget)
+      registerWidget(widget)
+    end
+
     def unregisterWidget(widget)
       if widget.kind_of?(PlasmaScripting::Applet)
         super(widget.applet_script.applet)
@@ -541,6 +553,10 @@ module Plasma
         super
       end
     end
+
+    def unregister_widget(widget)
+      unregisterWidget(widget)
+    end
   end
 
   class TreeView < Qt::Base
@@ -575,6 +591,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -601,6 +621,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -627,6 +651,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -653,6 +681,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -679,6 +711,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -705,6 +741,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -731,6 +771,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -757,6 +801,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -783,6 +831,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -809,6 +861,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -835,6 +891,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -861,6 +921,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -887,6 +951,10 @@ module Qt
       end
     end
 
+    def parent_item=(item)
+      setParentItem(item)
+    end
+
     def parentItem=(item)
       setParentItem(item)
     end
@@ -913,6 +981,10 @@ module Qt
       end
     end
 
+    def add_item(*args)
+      addItem(*args)
+    end
+
     def addItem(*args)
       sargs = []
       for i in 0...args.length do
@@ -933,6 +1005,10 @@ module Qt
       end
     end
 
+    def set_alignment(item, alignment)
+      setAlignment(item, alignment)
+    end
+
     def setAlignment(item, alignment)
       if item.kind_of?(PlasmaScripting::Applet)
         super(item.applet_script.applet, alignment)
@@ -955,6 +1031,10 @@ module Qt
       super(*sargs)
     end
 
+    def add_item(*args)
+      addItem(*args)
+    end
+
     def addItem(*args)
       sargs = []
       for i in 0...args.length do
@@ -975,6 +1055,10 @@ module Qt
       end
     end
 
+    def insert_item(index, item)
+      insertItem(index, item)
+    end
+
     def insertItem(index, item)
       if item.kind_of?(PlasmaScripting::Applet)
         super(index, item.applet_script.applet)
@@ -983,6 +1067,10 @@ module Qt
       end
     end
 
+    def set_alignment(item, alignment)
+      setAlignment(item, alignment)
+    end
+
     def setAlignment(item, alignment)
       if item.kind_of?(PlasmaScripting::Applet)
         super(item.applet_script.applet, alignment)
@@ -991,6 +1079,10 @@ module Qt
       end
     end
 
+    def set_stretch_factor(item, stretch)
+      setStretchFactor(item, stretch)
+    end
+
     def setStretchFactor(item, stretch)
       if item.kind_of?(PlasmaScripting::Applet)
         super(item.applet_script.applet, stretch)
@@ -999,6 +1091,10 @@ module Qt
       end
     end
 
+    def stretch_factor(item)
+      stretchFactor(item)
+    end
+
     def stretchFactor(item)
       if item.kind_of?(PlasmaScripting::Applet)
         super(item.applet_script.applet)
