@@ -63,8 +63,7 @@
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
-extern "C"
-{
+extern "C" {
 #include <X11/extensions/dpms.h>
     int __kde_do_not_unload = 1;
 
@@ -102,7 +101,8 @@ public:
             : notifier(Solid::Control::PowerManager::notifier())
             , battery(0)
             , currentConfig(0)
-            , status(PowerDevilDaemon::NoAction) {};
+            , status(PowerDevilDaemon::NoAction)
+            , ckSessionInterface(0) {};
 
     Solid::Control::PowerManager::Notifier * notifier;
     QPointer<Solid::Battery> battery;
@@ -128,6 +128,10 @@ public:
 
     int batteryPercent;
     bool isPlugged;
+
+    // ConsoleKit stuff
+    QDBusInterface *ckSessionInterface;
+    bool ckAvailable;
 };
 
 PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
@@ -165,6 +169,9 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
         return;
     }
 
+    // First things first: let's set up PowerDevil to be aware of active session
+    setUpConsoleKit();
+
     d->profilesConfig = KSharedConfig::openConfig("powerdevilprofilesrc", KConfig::SimpleConfig);
     setAvailableProfiles(d->profilesConfig->groupList());
 
@@ -176,7 +183,7 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
     d->ksmServerIface = new OrgKdeKSMServerInterfaceInterface("org.kde.ksmserver", "/KSMServer",
             QDBusConnection::sessionBus(), this);
 
-    /*  Not needed anymore; I am not sure if we will need that in a future, so I leave it here 
+    /*  Not needed anymore; I am not sure if we will need that in a future, so I leave it here
      *  just in case.
      *
      *   d->kscreenSaverIface = new OrgKdeScreensaverInterface("org.freedesktop.ScreenSaver", "/ScreenSaver",
@@ -196,7 +203,7 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
     if (PowerDevilSettings::pollingSystem() == -1) {
         // Ok, new configuration... so let's see what we've got!!
 
-        QMap<AbstractSystemPoller::PollingType, QString> pList = d->pollLoader->getAvailableSystems();
+        QHash<AbstractSystemPoller::PollingType, QString> pList = d->pollLoader->getAvailableSystems();
 
         if (pList.contains(AbstractSystemPoller::XSyncBased)) {
             PowerDevilSettings::setPollingSystem(AbstractSystemPoller::XSyncBased);
@@ -224,6 +231,7 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
 
 PowerDevilDaemon::~PowerDevilDaemon()
 {
+    delete d->ckSessionInterface;
     delete d;
 }
 
@@ -327,7 +335,7 @@ void PowerDevilDaemon::setUpPollingSystem()
 
 bool PowerDevilDaemon::loadPollingSystem(AbstractSystemPoller::PollingType type)
 {
-    QMap<AbstractSystemPoller::PollingType, QString> pList = d->pollLoader->getAvailableSystems();
+    QHash<AbstractSystemPoller::PollingType, QString> pList = d->pollLoader->getAvailableSystems();
 
     if (!pList.contains(type)) {
         return false;
@@ -351,10 +359,11 @@ QVariantMap PowerDevilDaemon::getSupportedPollingSystems()
 {
     QVariantMap map;
 
-    QMap<int, QString> pmap = d->pollLoader->getAvailableSystemsAsInt();
+    QHash<int, QString> pmap = d->pollLoader->getAvailableSystemsAsInt();
 
-    foreach(int ent, pmap.keys()) {
-        map[pmap[ent]] = ent;
+    QHash<int, QString>::const_iterator i;
+    for (i = pmap.constBegin(); i != pmap.constEnd(); ++i) {
+        map[i.value()] = i.key();
     }
 
     return map;
@@ -364,10 +373,14 @@ void PowerDevilDaemon::resumeFromIdle()
 {
     KConfigGroup * settings = getCurrentProfile();
 
-    Solid::Control::PowerManager::setBrightness(settings->readEntry("brightness").toInt());
-
     POLLER_CALL(d->pollLoader->poller(), stopCatchingIdleEvents());
     POLLER_CALL(d->pollLoader->poller(), forcePollRequest());
+
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
+    Solid::Control::PowerManager::setBrightness(settings->readEntry("brightness").toInt());
 }
 
 void PowerDevilDaemon::refreshStatus()
@@ -404,15 +417,15 @@ void PowerDevilDaemon::acAdapterStateChanged(int state, bool forced)
         emitNotification("unplugged", i18n("The power adaptor has been unplugged"));
     }
 
-    if (!forced)
+    if (!forced) {
         reloadProfile(state);
+    }
 
     applyProfile();
 }
 
 #ifdef HAVE_DPMS
-extern "C"
-{
+extern "C" {
     int dropError(Display *, XErrorEvent *);
     typedef int (*XErrFunc)(Display *, XErrorEvent *);
 }
@@ -425,10 +438,15 @@ int dropError(Display *, XErrorEvent *)
 
 void PowerDevilDaemon::applyProfile()
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     KConfigGroup * settings = getCurrentProfile();
 
-    if (!settings)
+    if (!settings) {
         return;
+    }
 
     Solid::Control::PowerManager::setBrightness(settings->readEntry("brightness").toInt());
     Solid::Control::PowerManager::setCpuFreqPolicy((Solid::Control::PowerManager::CpuFreqPolicy)
@@ -459,10 +477,15 @@ void PowerDevilDaemon::applyProfile()
 
 void PowerDevilDaemon::setUpDPMS()
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     KConfigGroup * settings = getCurrentProfile();
 
-    if (!settings)
+    if (!settings) {
         return;
+    }
 
 #ifdef HAVE_DPMS
 
@@ -535,8 +558,13 @@ void PowerDevilDaemon::batteryChargePercentChanged(int percent, const QString &u
 
     setBatteryPercent(charge);
 
-    if (Solid::Control::PowerManager::acAdapterState() == Solid::Control::PowerManager::Plugged)
+    if (Solid::Control::PowerManager::acAdapterState() == Solid::Control::PowerManager::Plugged) {
         return;
+    }
+
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
 
     if (charge <= PowerDevilSettings::batteryCriticalLevel()) {
         switch (PowerDevilSettings::batLowAction()) {
@@ -600,6 +628,10 @@ void PowerDevilDaemon::batteryChargePercentChanged(int percent, const QString &u
 
 void PowerDevilDaemon::buttonPressed(int but)
 {
+    if (!checkIfCurrentSessionActive() || d->screenSaverIface->GetActive()) {
+        return;
+    }
+
     KConfigGroup * settings = getCurrentProfile();
 
     if (!settings)
@@ -681,18 +713,30 @@ void PowerDevilDaemon::buttonPressed(int but)
 
 void PowerDevilDaemon::decreaseBrightness()
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     int currentBrightness = qMax(0, (int)(Solid::Control::PowerManager::brightness() - 10));
     Solid::Control::PowerManager::setBrightness(currentBrightness);
 }
 
 void PowerDevilDaemon::increaseBrightness()
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     int currentBrightness = qMin(100, (int)(Solid::Control::PowerManager::brightness() + 10));
     Solid::Control::PowerManager::setBrightness(currentBrightness);
 }
 
 void PowerDevilDaemon::shutdownNotification(bool automated)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (!d->lockHandler->setNotificationLock(automated)) {
         return;
     }
@@ -709,6 +753,10 @@ void PowerDevilDaemon::shutdownNotification(bool automated)
 
 void PowerDevilDaemon::suspendToDiskNotification(bool automated)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (!d->lockHandler->setNotificationLock(automated)) {
         return;
     }
@@ -725,6 +773,10 @@ void PowerDevilDaemon::suspendToDiskNotification(bool automated)
 
 void PowerDevilDaemon::suspendToRamNotification(bool automated)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (!d->lockHandler->setNotificationLock(automated)) {
         return;
     }
@@ -741,6 +793,10 @@ void PowerDevilDaemon::suspendToRamNotification(bool automated)
 
 void PowerDevilDaemon::standbyNotification(bool automated)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (!d->lockHandler->setNotificationLock(automated)) {
         return;
     }
@@ -757,6 +813,10 @@ void PowerDevilDaemon::standbyNotification(bool automated)
 
 void PowerDevilDaemon::shutdown(bool automated)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (!d->lockHandler->setJobLock(automated)) {
         return;
     }
@@ -775,6 +835,10 @@ void PowerDevilDaemon::shutdownDialog()
 
 void PowerDevilDaemon::suspendToDisk(bool automated)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (!d->lockHandler->setJobLock(automated)) {
         return;
     }
@@ -795,6 +859,10 @@ void PowerDevilDaemon::suspendToDisk(bool automated)
 
 void PowerDevilDaemon::suspendToRam(bool automated)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (!d->lockHandler->setJobLock(automated)) {
         return;
     }
@@ -814,6 +882,10 @@ void PowerDevilDaemon::suspendToRam(bool automated)
 
 void PowerDevilDaemon::standby(bool automated)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (!d->lockHandler->setJobLock(automated)) {
         return;
     }
@@ -856,7 +928,11 @@ void PowerDevilDaemon::poll(int idle)
      * We make an intensive use of qMin/qMax here to determine the minimum time.
      */
 
-   // kDebug() << "Polling started, idle time is" << idle << "seconds";
+    // kDebug() << "Polling started, idle time is" << idle << "seconds";
+
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
 
     KConfigGroup * settings = getCurrentProfile();
 
@@ -866,7 +942,7 @@ void PowerDevilDaemon::poll(int idle)
 
     if (!settings->readEntry("dimOnIdle", false) && !settings->readEntry("turnOffIdle", false) &&
             settings->readEntry("idleAction").toInt() == None) {
-     //   kDebug() << "Stopping timer";
+        //   kDebug() << "Stopping timer";
         POLLER_CALL(d->pollLoader->poller(), stopCatchingTimeouts());
         return;
     }
@@ -891,13 +967,13 @@ void PowerDevilDaemon::poll(int idle)
         minTime = qMin(minTime, minDimTime);
     }
 
-   // kDebug() << "Minimum time is" << minTime << "seconds";
+    // kDebug() << "Minimum time is" << minTime << "seconds";
 
     if (idle < minTime) {
         d->status = NoAction;
         int remaining = minTime - idle;
         POLLER_CALL(d->pollLoader->poller(), setNextTimeout(remaining * 1000));
-     //   kDebug() << "Nothing to do, next event in" << remaining << "seconds";
+        //   kDebug() << "Nothing to do, next event in" << remaining << "seconds";
         return;
     }
 
@@ -1009,6 +1085,10 @@ void PowerDevilDaemon::setUpNextTimeout(int idle, int minDimEvent)
 
 void PowerDevilDaemon::lockScreen()
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     emitNotification("doingjob", i18n("The screen is being locked"));
     d->screenSaverIface->Lock();
 }
@@ -1136,6 +1216,10 @@ KConfigGroup * PowerDevilDaemon::getCurrentProfile(bool forcereload)
 
 void PowerDevilDaemon::reloadProfile(int state)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (!recacheBatteryPointer()) {
         setCurrentProfile(PowerDevilSettings::aCProfile());
     } else {
@@ -1186,6 +1270,10 @@ void PowerDevilDaemon::reloadProfile(int state)
 
 void PowerDevilDaemon::setProfile(const QString & profile)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     setCurrentProfile(profile);
 
     /* Don't call refreshStatus() here, since we don't want the predefined profile
@@ -1202,6 +1290,10 @@ void PowerDevilDaemon::setProfile(const QString & profile)
 
 void PowerDevilDaemon::reloadAndStream()
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     reloadProfile();
 
     setAvailableProfiles(d->profilesConfig->groupList());
@@ -1274,16 +1366,28 @@ QStringList PowerDevilDaemon::getSupportedSchemes()
 
 void PowerDevilDaemon::setPowersavingScheme(const QString &scheme)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     Solid::Control::PowerManager::setScheme(scheme);
 }
 
 void PowerDevilDaemon::setGovernor(int governor)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     Solid::Control::PowerManager::setCpuFreqPolicy((Solid::Control::PowerManager::CpuFreqPolicy) governor);
 }
 
 void PowerDevilDaemon::suspend(int method)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     switch ((IdleAction) method) {
     case S2Disk:
         QTimer::singleShot(100, this, SLOT(suspendToDisk()));
@@ -1301,6 +1405,10 @@ void PowerDevilDaemon::suspend(int method)
 
 void PowerDevilDaemon::setBrightness(int value)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (value == -2) {
         // Then set brightness to half the current brightness.
 
@@ -1332,6 +1440,10 @@ void PowerDevilDaemon::turnOffScreen()
 
 void PowerDevilDaemon::profileFirstLoad()
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     KConfigGroup * settings = getCurrentProfile();
 
     if (!settings)
@@ -1411,6 +1523,10 @@ void PowerDevilDaemon::setACPlugged(bool newplugged)
 
 void PowerDevilDaemon::setCurrentProfile(const QString &profile)
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
     if (profile != d->currentProfile) {
         d->currentProfile = profile;
         profileFirstLoad();
@@ -1424,4 +1540,62 @@ void PowerDevilDaemon::setAvailableProfiles(const QStringList &aProfiles)
     emit profileChanged(d->currentProfile, d->availableProfiles);
 }
 
+bool PowerDevilDaemon::checkIfCurrentSessionActive()
+{
+    if (!d->ckAvailable) {
+        // No way to determine if we are on the current session, simply suppose we are
+        kDebug() << "Can't contact ck";
+        return true;
+    }
+
+    QDBusReply<bool> rp = d->ckSessionInterface->call("IsActive");
+
+    return rp.value();
+}
+
+void PowerDevilDaemon::setUpConsoleKit()
+{
+    // Let's cache the needed information to check if our session is actually active
+
+    if (!QDBusConnection::systemBus().interface()->isServiceRegistered("org.freedesktop.ConsoleKit")) {
+        // No way to determine if we are on the current session, simply suppose we are
+        kDebug() << "Can't contact ck";
+        d->ckAvailable = false;
+        return;
+    } else {
+        d->ckAvailable = true;
+    }
+
+    // Otherwise, let's ask ConsoleKit
+    QDBusInterface ckiface("org.freedesktop.ConsoleKit", "/org/freedesktop/ConsoleKit/Manager",
+                           "org.freedesktop.ConsoleKit.Manager", QDBusConnection::systemBus());
+
+    QDBusReply<QDBusObjectPath> sessionPath = ckiface.call("GetCurrentSession");
+
+    if (!sessionPath.isValid() || sessionPath.value().path().isEmpty()) {
+        kDebug() << "The session is not registered with ck";
+        d->ckAvailable = false;
+        return;
+    }
+
+    d->ckSessionInterface = new QDBusInterface("org.freedesktop.ConsoleKit", sessionPath.value().path(),
+            "org.freedesktop.ConsoleKit.Session", QDBusConnection::systemBus());
+
+    if (!d->ckSessionInterface->isValid()) {
+        // As above
+        kDebug() << "Can't contact iface";
+        d->ckAvailable = false;
+        return;
+    }
+
+    /* If everything's fine, let's attach ourself to the ActiveChanged signal.
+     * You'll see we're attaching to refreshStatus without any further checks. You know why?
+     * refreshStatus already checks if the console is active, so the check is already happening
+     * in the underhood
+     */
+
+    QDBusConnection::systemBus().connect("org.freedesktop.ConsoleKit", sessionPath.value().path(),
+                                         "org.freedesktop.ConsoleKit.Session", "ActiveChanged", this, SLOT(refreshStatus()));
+}
+
 #include "PowerDevilDaemon.moc"
